import gleam/float
import gleam/int
import gleam/io
import gleam/list
import gleam/result
import gleam/string
import kicad_sexpr/decode
import kicad_sexpr/parse
import kicad_sexpr/token
import simplifile

fn print_stats(title: String, successes: Int, total: Int) {
  io.println(
    title
    <> ": "
    <> int.to_string(successes)
    <> "/"
    <> int.to_string(total)
    <> " ("
    <> int.to_float(100 * successes) /. int.to_float(total)
    |> float.to_precision(2)
    |> float.to_string
    |> string.append("%")
    <> ")",
  )
}

pub fn main() -> Nil {
  let assert Ok(file_names) =
    simplifile.get_files("/usr/share/kicad/footprints")

  // let #(file_names, _) = file_names |> list.drop(0) |> list.split(1000)
  // let file_names = list.sample(file_names, 1000)
  // let file_names = ["/usr/share/kicad/symbols/RF_Module.kicad_sym"]

  let num_file_names = list.length(file_names)
  io.println("Total: " <> int.to_string(num_file_names))

  let #(successfully_read, _failed_to_read) =
    file_names
    |> list.map(fn(file_name) {
      simplifile.read_bits(file_name)
      |> result.map(fn(res) { #(file_name, res) })
      |> result.map_error(fn(res) { #(file_name, res) })
    })
    |> result.partition
  let num_successfully_read = list.length(successfully_read)
  print_stats("Read", num_successfully_read, num_file_names)

  let #(successfully_parsed, _failed_to_parse) =
    successfully_read
    |> list.map(fn(data) {
      let #(file_name, file_contents) = data
      parse.run(file_contents)
      |> result.map(fn(res) { #(file_name, file_contents, res) })
      |> result.map_error(fn(res) { #(file_name, file_contents, res) })
    })
    |> result.partition
  let num_successfully_parsed = list.length(successfully_parsed)
  print_stats("Parsed", num_successfully_parsed, num_successfully_read)

  let #(successfully_decoded, failed_to_decode) =
    successfully_parsed
    |> list.map(fn(data) {
      let #(file_name, file_contents, sexpr) = data
      decode.run(token.footprint_file, sexpr)
      |> result.map(fn(res) { #(file_name, file_contents, sexpr, res) })
      |> result.map_error(fn(res) { #(file_name, file_contents, sexpr, res) })
    })
    |> result.partition
  let num_successfully_decoded = list.length(successfully_decoded)
  print_stats("Decoded", num_successfully_decoded, num_successfully_parsed)
  list.each(failed_to_decode, fn(data) {
    let #(file_name, _file_contents, _sexpr, error) = data
    io.println(file_name)
    echo error
    // panic
  })

  // let assert Ok(footprint_str) =
  //   simplifile.read(
  //     "/usr/share/kicad/footprints/Relay_SMD.pretty/Relay_DPDT_AXICOM_IMSeries_JLeg.kicad_mod",
  //   )
  // // let assert Ok(footprint_str) = simplifile.read("test2.kicad_mod")
  // let assert Ok(t) =
  //   parse.run(
  //     " (wobble \"hi there \\\" \\\\ this is a test\" (wibble foo bar) bar) ",
  //   )
  // io.println(parse.sexpr_to_pretty_string(t))
  // let assert Ok(t) = decode.run(wobble, t)
  // echo t
  Nil
}
